#!/usr/bin/python
##############################################################################
#
# Copyright (C) Charles Shapiro 2006 charles.shapiro@tomshiro.org
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#
# Manage filenames. Write new files only for unique sudoku//palettes.
#
# $Id: filer.py,v 1.5 2006/05/25 02:55:29 devel Exp $
#
##################################################################################
import os
import re
import glob
import sys


class filer:
	"""Manage filenames for sudoku, palettes."""

	def setup(self):
		"""Set myself up with null values."""
		self.dirname=""
		self.fglob=""
		self.fname=""
		self.sumFile=""

	def get_next_file(self,md5Str):
		"""Get the next filename and tell whether it is a duplicate file. 
                Entry Point."""
		retFN=""
		isDup=False
		retNum = self.get_dup_num(md5Str)
		if retNum > -1:
			isDup=True
			retFN=str(retNum)+"."+self.fname
		else:
			retNum=self.get_last_file_name()
			retNum += 1
			retFN=str(retNum)+"."+self.fname
		retVal=(retFN,isDup)
		return retVal
							

	def __init__(self,dirname,fglob):
		"""Create myself."""
		self.setup()
		self.dirname=dirname
		self.fglob=fglob
		self.fname=fglob[2:]
		self.sumFile=os.path.join(self.dirname,self.fname+".sums")
	
	def fname_compare(self,a,b):
		"""Compare a pair of numeric filenames, discover the largest."""
		int_a=self.num_from_fn(a)
		int_b=self.num_from_fn(b)
		retVal=0
		if int_a > int_b:
			retVal=1
		if int_a < int_b:
			retVal=-1
		return retVal

	def get_dup_num(self,md5Str):
		"""Get next valid file number (may be a duplicate file)"""
		retVal=0
		md5Dict={}
		try:
			listFile=file(self.sumFile,"r")
		except IOError:
			self.make_new_listfile()
			retVal=self.get_dup_num(md5Str)
		if retVal == 0:
			for line in listFile.readlines():
				workList=line.strip().split()
				md5Dict[workList[0]]=workList[1]
			listFile.close()
			if md5Dict.has_key(md5Str):
				retVal=self.num_from_fn(md5Dict[md5Str])
			else:
				retVal=-1
		return retVal
				
	def add_sum_to_listfile(self,md5Str,fn):
		"""Add an md5sum to listfile."""
		listFile=file(self.sumFile,"a")
                listFile.write(md5Str+"  "+fn+"\n")
		listFile.close()

	def make_new_listfile(self):
		"""Make a list of files and md5sums in this directory. Must 
		throw IOERR exception if it fails to avoid recursion loop in 
		get_dup_filenum()."""
		cmdLine="cd " + self.dirname +" ; " + "/usr/bin/md5sum " + self.fglob + " > " + os.path.basename(self.sumFile)
		os.system(cmdLine)
		if not os.path.exists(self.sumFile):
			raise(IOError(cmdLine))
			
									
	def num_from_fn(self,fnstr):
		"""Translate a filename into a file number."""
		retVal=re.compile("^[0-9]+").search(fnstr).group()
		if retVal == None:
			retVal=0
		else:
			retVal=int(retVal)
		return retVal

	def get_last_file_name(self):
		"""Get the next number from a directory full of files."""
		file_list=glob.glob(os.path.join(self.dirname,self.fglob))
		retVal=0
		if len(file_list):
			right_file_list=[os.path.basename(elem) for elem in file_list]
			right_file_list.sort(self.fname_compare)
			theLast=right_file_list.pop()
			retVal=self.num_from_fn(theLast)
		return int(retVal)

if __name__ == "__main__":
	myFiler=filer("/tmp/foo","*.sud")
	print myFiler.get_next_file(sys.argv[1])
	
