#!/usr/bin/python
#####################################################################
#
# Sudoku object.
#
#
#####################################################################
import string
import palette

class A_Sudoku:
	def __init__(self,inStr):
		""" Make myself from input string."""
		self.Valid=0
		self.solutionBreak=""
		self.numberList=self.fromString(inStr)

	def is_valid_group(self,numList,title):
		""" Is the passed list of numbers a valid part of a sudoku?
		That is, does it contain no duplicate digits?"""
		checkDict={}
		retVal=0
		for ii in numList:
			checkDict[ii]="check"
		if len(checkDict) == len(numList):
			retVal=1
		else:
			self.solutionBreak=title
		return retVal
			
	def is_solution(self):
		""" Is this object a valid solution?"""
		retVal=1
		if not self.Valid:
			self.solutionBreak="Not enough numbers"
			retVal=0
		if retVal == 1:
			for ii in range(0,9):
				retVal = self.is_valid_group(self.numberList[ii],"Row " + str(ii+1))
				if retVal == 0:
					break
		if retVal == 1:
			for ii in range(0,9):
				colList=[]
				for jj in range(0,9):
					colList.append(self.numberList[jj][ii])
				retVal=self.is_valid_group(colList,"Column " + str(ii+1))
				if retVal == 0:
					break;
		if retVal == 1:
			for ii in range(0,9,3):
				blockList=[]
				blockList.extend(self.numberList[ii][0:3])
				blockList.extend(self.numberList[ii+1][0:3])
				blockList.extend(self.numberList[ii+2][0:3])
				retVal=self.is_valid_group(blockList,"Left block " + str(ii+1))
				if retVal == 0:
					break;
		if retVal == 1:
			for ii in range(0,9,3):
				blockList=[]
				blockList.extend(self.numberList[ii][3:6])
				blockList.extend(self.numberList[ii+1][3:6])
				blockList.extend(self.numberList[ii+2][3:6])
				retVal=self.is_valid_group(blockList,"Middle block " + str(ii+1))
				if retVal == 0:
					break;
		if retVal == 1:
			for ii in range(0,9,3):
				blockList=[]
				blockList.extend(self.numberList[ii][6:9])
				blockList.extend(self.numberList[ii+1][6:9])
				blockList.extend(self.numberList[ii+2][6:9])
				retVal=self.is_valid_group(blockList,"Right block " + str(ii+1))
				if retVal == 0:
					break;
		return retVal

	def __str__(self):
		""" Turn me into a string. """
		outStr=""
		onest=twost=third=fourth=fifth=sixth=seventh=eighth=nineth=0
		for ii in range(0,9):
			(onest,twost,third,fourth,fifth,sixth,seventh,eighth,nineth)=self.numberList[ii]
			outStr += ("%d %d %d %d %d %d %d %d %d\n" % (onest,twost,third,fourth,fifth,sixth,seventh,eighth,nineth))
		return outStr

	def fromString(self,inptStr):
		""" Create number list from an input string containing random
		stuff plus at least 81 digits"""	
		inptStrXlateList=[' ' for elem in range(0,255)]
		for ii in range(49,58):
			inptStrXlateList[ii]=chr(ii)
		inptStrXlateTable="".join(inptStrXlateList)
		numberspaces=inptStr.translate(inptStrXlateTable)
		nspaceList=numberspaces.split(" ")
		numbers="".join(nspaceList)
		numbers="%.81s" % (numbers)
		numberList=[]
		if len(numbers) >= 81:
			topList=81
		else:
			topList=len(numbers)
		for ii in range(0,topList):
		 	numberList.append(string.atoi(numbers[ii]))
		self.Valid=1
		if len(numberList) < 81:
			self.Valid=0
			for ii in range(len(numberList),81):
				numberList.append(0)
		retNumberList=[]
		rowNumList=[]
		for ii in range(0,81,9):
			rowNumList=numberList[ii:ii+9]
			retNumberList.append(rowNumList)
		print "self.Valid: " + str(self.Valid)
		return retNumberList

	def color_list(self,palette)
		"""Return a list of the colored-in sudoku."""
		retVal=[]
		
		if not self.Valid:
			for ii in range(0,81)
				retVal.append("black")
		if self.Valid:
			for ii in self.numberList:
				row_list=[palette.color_for_number(memb) for memb in ii]
				retVal.append(row_list)
		return retVal
			
							
#
# Testing
#
import sys
if __name__ == "__main__":
	inFile=open(sys.argv[1])
	inptStr=inFile.read()
	inFile.close
	mySudoku = A_Sudoku(inptStr)
	print mySudoku
	if mySudoku.is_solution():
		print "This is a valid solution"
	else:
		print "Not a valid solution: " + mySudoku.solutionBreak

