#!/usr/bin/python
#####################################################################
#
# Copyright (C) Charles Shapiro 2006 charles.shapiro@tomshiro.org
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
# Sudoku object.
#
# $Id: sudoku.py,v 1.12 2007/01/04 02:22:26 devel Exp $
#
#####################################################################
import string
import palette
import md5
import copy

class A_Sudoku:
	def __init__(self,inStr,is_file_name=False):
		""" Make myself from input string."""
		self.Valid=0
		self.solutionBreak=""
		if is_file_name:
			infile=file(inStr)
			self.numberList=self.fromString(infile.read())
			infile.close()
		else:
			self.numberList=self.fromString(inStr)


	def is_valid_group(self,numList,title):
		""" Is the passed list of numbers a valid part of a sudoku?
		That is, does it contain no duplicate digits?"""
		checkDict={}
		retVal=0
		for ii in numList:
			checkDict[ii]="check"
		if len(checkDict) == len(numList):
			retVal=1
		else:
			self.solutionBreak=title
		return retVal

	def __eq__(self,other):
		"""Am I equal to other?"""		
		retVal = True
	 	for ii in range(0,9):
			if self.numberList[ii] != other.numberList[ii]:
				retVal=False
		return retVal

	def normalize(self,group,num,type):
		"""Normalize myself given group on given type at given num.
		type can be 'row','col',or 'block'.  num is which one is 
		to be normalized. group is what we normalize to."""
		if type == "row":
			for ii in range(0,9):
				self.swap_num(group[ii],self.numberList[num][ii])
		if type == "block":
			for ii in range(0,9):
				self.swap_num(group[ii],self.get_block(num)[ii])
		if type == "col":
			for ii in range(0,9):
				self.swap_num(group[ii],self.numberList[ii][num])

	def swap_num(self,fr,to):
		"""Swap a pair of numbers throughout a sudoku."""
		new_numlist=[]
		new_row=[]
		jj=0
		for ii in self.numberList:
			for kk in ii:
				jj=kk
				if kk == fr:
					jj=to
				if kk == to:
					jj=fr
				new_row.append(jj)
			new_numlist.append(new_row)
			new_row=[]
		self.numberList[:]=new_numlist
			
	def get_hash(self):
		"""Return hash of sudoku contents. This must exactly match
                the output of md5sum(1) or much wacky fun will ensue."""
		mdh=md5.new()
		mdh.update(str(self))
		retVal=mdh.hexdigest()
		return retVal	
		
	def get_cell(self,cellno):
		"""Get an individual cell by sequential number."""
		rownum=cellno / 9
		colnum=cellno % 9
		the_row=self.get_row(rownum)
		retVal=the_row[colnum]
		return retVal
		
	def get_col(self,num):
		"""Get a given column from the sudoku."""
		retVal=[]
		for jj in range(0,9):
			retVal.append(self.numberList[jj][num])
		return retVal

	def get_row(self,num):
		"""Get a given row from the sudoku."""
		retVal=self.numberList[num]
		return retVal

	def get_block(self,num):
		"""Return list of a block."""
		blockList=[(0,(0,3)),(0,(3,6)),(0,(6,9)),
(3,(0,3)),(3,(3,6)),(3,(6,9)),
(6,(0,3)),(6,(3,6)),(6,(6,9))]
		horiz_tuple=blockList[num][1]
		my_row=blockList[num][0]
		retVal=[]

	 	retVal.extend(self.numberList[my_row][horiz_tuple[0]:horiz_tuple[1]])
		retVal.extend(self.numberList[my_row+1][horiz_tuple[0]:horiz_tuple[1]])
		retVal.extend(self.numberList[my_row+2][horiz_tuple[0]:horiz_tuple[1]])
	 	return retVal

		
	def is_solution(self):
		""" Is this object a valid solution?"""
		retVal=1
		if not self.Valid:
			self.solutionBreak="Not enough numbers"
			retVal=0
		if retVal == 1:
			for ii in range(0,9):
				retVal = self.is_valid_group(self.get_row(ii),"Row " + str(ii+1))
				if retVal == 0:
					break
		if retVal == 1:
			for ii in range(0,9):
				retVal=self.is_valid_group(self.get_col(ii),"Column " + str(ii+1))
				if retVal == 0:
					break;
		if retVal == 1:
			for ii in range(0,9):
				blockList=self.get_block(ii)
				if not self.is_valid_group(blockList,"Block %d" % (ii)):
						retVal=0
						
		return retVal

	def __add__(self,other):		
		"""Add a pair of sudoku to get a third."""
		adder=copy.deepcopy(self)
		for ii in range(0,9):
			adder.normalize(other.get_row(ii),ii,"row")
		for ii in range(0,9):
			adder.normalize(other.get_col(ii),ii,"col")
		return adder

	def __str__(self):
		""" Turn me into a string. """
		outStr=""
		onest=twost=third=fourth=fifth=sixth=seventh=eighth=nineth=0
		for ii in range(0,9):
			(onest,twost,third,fourth,fifth,sixth,seventh,eighth,nineth)=self.numberList[ii]
			outStr += ("%d %d %d %d %d %d %d %d %d\n" % (onest,twost,third,fourth,fifth,sixth,seventh,eighth,nineth))
		return outStr

	def save_to_file(self,fileName):
	        """Save myself to a file."""
		my_str=str(self)
		outfile=file(fileName,"w")
		outfile.write(my_str)
		outfile.close()

	def fromString(self,inptStr):
		""" Create number list from an input string containing random
		stuff plus at least 81 digits"""	
		inptStrXlateList=[' ' for elem in range(0,256)]
		for ii in range(49,58):
			inptStrXlateList[ii]=chr(ii)
		inptStrXlateTable="".join(inptStrXlateList)
		numberspaces=inptStr.translate(inptStrXlateTable)
		nspaceList=numberspaces.split(" ")
		numbers="".join(nspaceList)
		numbers="%.81s" % (numbers)
		numberList=[]
		if len(numbers) >= 81:
			topList=81
		else:
			topList=len(numbers)
		for ii in range(0,topList):
		 	numberList.append(string.atoi(numbers[ii]))
		self.Valid=1
		if len(numberList) < 81:
			self.Valid=0
			for ii in range(len(numberList),81):
				numberList.append(0)
		retNumberList=[]
		rowNumList=[]
		for ii in range(0,81,9):
			rowNumList=numberList[ii:ii+9]
			retNumberList.append(rowNumList)
		return retNumberList

	def color_list(self,palette):
		"""Return a list of the colored-in sudoku."""
		retVal=[]
		my_numberlist=[]
		my_numberlist[:]=self.numberList

		if len(my_numberlist) < 81:
			numAppend = 81-len(my_numberlist)
			for kk in range(numAppend,0,-1):
				my_numberlist.append("0")
		for ii in self.numberList:
			row_list=[(memb,palette.color_for_number(memb)) for memb in ii]
			retVal.append(row_list)
		return retVal
			
							
#
# Testing
#
import sys
if __name__ == "__main__":
	OneSudoku = A_Sudoku(sys.argv[1],True)
	print OneSudoku
	TwoSudoku = A_Sudoku(sys.argv[2],True)
	print TwoSudoku
	ThreeSudoku=OneSudoku + TwoSudoku
	print "Added sudoku is solution: " + str(ThreeSudoku.is_solution())
	print ThreeSudoku
	print ("One Sudoku:" )
	print OneSudoku	
	print OneSudoku.get_cell(3)
        print OneSudoku.get_cell(19)

	

