#!/usr/bin/python
#######################################################################
#
# Copyright (C) Charles Shapiro 2006 charles.shapiro@tomshiro.org
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
# Template grinder. Given a template string and a dictionary of 
# substitute names + values,
# replaces each delimited macro in the string with the value.
# Macros are delimited with "@@name@@".
#
# $Id: templategrinder.py,v 1.2 2006/04/28 03:17:52 devel Exp $
#
#######################################################################
import re
import os

class templateGrinder:
	def setup(self):
		self.template_list=[]
		self.template_dict={}

	def template_from_fil(self,filename):
		"""Read template from a filename."""
		try:
			fil=file(filename,"r")
			self.set_str(fil.read())
			fil.close()
		except IOError:
			self.set_str("No Template")
	
	def __init__(self,template_dict,template_str,isFile=False):
		"""Create myself from a dictionary and string."""
		self.setup()
		if isFile:
			self.template_from_fil(template_str)
		else:
			self.set_str(template_str)
		
		self.set_dict(template_dict)
			
	def set_str(self,tStr):
		"""Make input template into a list we can pop() from
		top to bottom."""
		self.template_list=tStr.split('\n')

	def set_dict(self,tDict):
		""" Set template dictionary member"""
		for k in tDict.keys():
			srchStr="@@" + k + "@@"
			reSrch=re.compile(srchStr)
			self.template_dict[reSrch]=tDict[k]

	def process_line(self,line):
		""" Substitute all macros in a single line."""
		for srch in self.template_dict.keys():
			substitute=self.template_dict[srch]
			found=srch.search(line)			
			while found:
				theSpan=found.span()
				lineBottom=line[0:theSpan[0]]
				lineTop=line[theSpan[1]:len(line)]
				line=lineBottom + substitute + lineTop
				found=srch.search(line)
		line += os.linesep
		return line

	def grind(self):
		"""Apply template dictionary to template string to produce
		result string."""
		success=True
		result_str=""
		if len(self.template_list) == 0 or len(self.template_dict) == 0:
			success=False
		if success == True:
			myList=self.template_list
			for the_line in myList:
				result_str += self.process_line(the_line)
		return result_str

			
		
if __name__ == "__main__":
	import sys
	skelFN=sys.argv[1]
	macFN=sys.argv[2]

	thefil=file(macFN,'r')
	macList=thefil.readlines()
	macList=[memb.strip() for memb in macList]
	thefil.close()
	macDict=dict([member.split(":") for member in macList])

	my_grinder=templateGrinder(macDict,skelFN,True)
	print my_grinder.grind()
	
	